@@ -331,6 +331,8 @@ static int dbd_db_connect( SV *dbh,
 {
   D_imp_drh_from_dbh;
   SQLRETURN ret;
+  SQLCHAR *new_dsn = NULL;
+  int dsn_length;
 
   imp_dbh->hdbc = SQL_NULL_HDBC;
 
@@ -379,8 +381,25 @@ static int dbd_db_connect( SV *dbh,
     }
   }
 
-  ret = SQLConnect(imp_dbh->hdbc,dbname,SQL_NTS,uid,SQL_NTS,pwd,SQL_NTS);
-  ret = check_error( dbh, ret, "Connect failed" );
+  /* If the string contains a =, use SQLDriverConnect */
+  if (strstr (dbname, "=") != NULL) {
+      if (uid != NULL && strlen(uid) > 0){
+          if (strstr(dbname, ";uid=") == NULL && strstr(dbname, ";UID=") == NULL){
+              dsn_length = strlen(dbname) + strlen(uid) + strlen(pwd) + sizeof(";UID=;PWD=;")+1;
+              new_dsn = (char *)malloc(sizeof(char)*dsn_length);
+              ret = check_error( dbh, ret, "Unable to allocate DSN string" );
+              sprintf(new_dsn, "%s;UID=%s;PWD=%s;",dbname,uid,pwd);
+              dbname = new_dsn;
+          }
+      }
+      ret = SQLDriverConnect(imp_dbh->hdbc,(SQLHWND)NULL,dbname,SQL_NTS,NULL,0,NULL,SQL_DRIVER_NOPROMPT);
+      ret = check_error( dbh, ret, "Connect failed" );
+      if (new_dsn != NULL) 
+          free((void *)new_dsn);
+  } else {
+      ret = SQLConnect(imp_dbh->hdbc,dbname,SQL_NTS,uid,SQL_NTS,pwd,SQL_NTS);
+      ret = check_error( dbh, ret, "Connect failed" );
+  }
   if( SQL_SUCCESS != ret )
     goto exit;
 
@@ -786,6 +805,8 @@ int dbd_db_STORE_attrib( SV *dbh,
 {
   STRLEN kl;
   char *key = SvPV( keysv, kl );
+  char setSchemaSQL[BUFSIZ];
+  SQLHSTMT hstmt;
   SQLINTEGER Attribute = getConnectAttr( key, kl );
   SQLRETURN ret;
 #ifndef AS400
@@ -823,6 +844,19 @@ int dbd_db_STORE_attrib( SV *dbh,
 #ifndef AS400
     case SQL_ATTR_CLISCHEMA:
     case SQL_ATTR_CURRENT_SCHEMA:
+      if( SvOK( valuesv ) )
+      {
+        STRLEN vl;
+        ValuePtr = (SQLPOINTER)SvPV( valuesv, vl );
+        StringLength = (SQLINTEGER)vl;
+      }
+
+      ret = SQLAllocHandle(SQL_HANDLE_STMT, imp_dbh->hdbc, &hstmt);
+          sprintf(setSchemaSQL, "SET CURRENT SCHEMA = '%s'", ValuePtr);
+      ret = SQLExecDirect(hstmt, (SQLCHAR *)setSchemaSQL, SQL_NTS);
+      SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
+      break;
+
     case SQL_ATTR_INFO_ACCTSTR:
     case SQL_ATTR_INFO_APPLNAME:
     case SQL_ATTR_INFO_USERID:
@@ -874,6 +908,14 @@ int dbd_db_STORE_attrib( SV *dbh,
     default:
       return FALSE;
   }
+
+  if ( Attribute != SQL_ATTR_CURRENT_SCHEMA) {
+    ret = SQLSetConnectAttr( imp_dbh->hdbc,
+                             Attribute,
+                             ValuePtr,
+                             StringLength );
+  }
+
   ret = SQLSetConnectAttr( imp_dbh->hdbc,
                            Attribute,
                            ValuePtr,
@@ -1776,245 +1818,308 @@ SV *dbd_db_get_info( SV        *dbh,
 
    memset( &buffer, '\0', sizeof( buffer ) );
 
-   ret = SQLGetInfo( imp_dbh->hdbc,
-                     infoType,
-                     valuePtr,
-                     bufferLength,
-                     &stringLength );
-
-   if( ret == SQL_SUCCESS_WITH_INFO &&  bufferLength < (stringLength + 1) )
-   {
-      if( DBIS->debug >= 2)
-      {
-         PerlIO_printf( DBILOGFP,
-            "GetInfo(%d) local buffer isn't big enough. stringlenght=%d\n",
-            infoType, stringLength );
-      }
-
-      /* Local buffer isn't big enough, dynamically allocate new one */
-      bufferLength = stringLength + 1;
-      Newc( 1, valuePtr, bufferLength, char, SQLPOINTER );
-      Zero( valuePtr, bufferLength, char );
-
-      ret = SQLGetInfo( imp_dbh->hdbc,
-                        infoType,
-                        valuePtr,
-                        bufferLength,
-                        &stringLength );
-   }
-
-   ret = check_error( dbh, ret, "Error calling SQLGetInfo" );
-
    /* Create our scalar value to return to app */
-   if( ret == SQL_SUCCESS )
+   switch( infoType )
    {
-      switch( infoType )
-      {
-         /* Create a scalar to hold the string infoTypes */
-         case SQL_DBMS_NAME:
-         case SQL_DBMS_VER:
-         case SQL_CATALOG_NAME:
-         case SQL_CATALOG_NAME_SEPARATOR:
-         case SQL_ACCESSIBLE_PROCEDURES:
-         case SQL_ACCESSIBLE_TABLES:
-         case SQL_CATALOG_TERM:
-         case SQL_COLLATION_SEQ:
-         case SQL_COLUMN_ALIAS:
-         case SQL_DATA_SOURCE_NAME:
-         case SQL_DATA_SOURCE_READ_ONLY:
-         case SQL_DATABASE_NAME:
-         case SQL_DESCRIBE_PARAMETER:
-         case SQL_DRIVER_NAME:
-         case SQL_DRIVER_ODBC_VER:
-         case SQL_DRIVER_VER:
-         case SQL_EXPRESSIONS_IN_ORDERBY:
-         case SQL_IDENTIFIER_QUOTE_CHAR:
-         case SQL_INTEGRITY:
-         case SQL_KEYWORDS:
-         case SQL_LIKE_ESCAPE_CLAUSE:
-         case SQL_MULT_RESULT_SETS:
-         case SQL_MULTIPLE_ACTIVE_TXN:
-         case SQL_NEED_LONG_DATA_LEN:
-         case SQL_ODBC_VER:
-         case SQL_ORDER_BY_COLUMNS_IN_SELECT:
-         case SQL_OUTER_JOINS:
-         case SQL_PROCEDURE_TERM:
-         case SQL_PROCEDURES:
-         case SQL_ROW_UPDATES:
-         case SQL_SCHEMA_TERM:
-         case SQL_SEARCH_PATTERN_ESCAPE:
-         case SQL_SERVER_NAME:
-         case SQL_SPECIAL_CHARACTERS:
-         case SQL_TABLE_TERM:
-         case SQL_USER_NAME:
-         case SQL_XOPEN_CLI_YEAR:
-
-            retsv = sv_2mortal( newSVpvn( (char*) valuePtr, (int)stringLength ) );
-            break;
-
-         /* Create a scalar to hold the 16-bit integer */
-         case SQL_CATALOG_LOCATION:
-         case SQL_CONCAT_NULL_BEHAVIOR:
-         case SQL_CORRELATION_NAME:
-         case SQL_CURSOR_COMMIT_BEHAVIOR:
-         case SQL_CURSOR_ROLLBACK_BEHAVIOR:
-         case SQL_FILE_USAGE:
-         case SQL_GROUP_BY:
-         case SQL_IDENTIFIER_CASE:
-         case SQL_MAX_CATALOG_NAME_LEN:
-         case SQL_MAX_COLUMN_NAME_LEN:
-         case SQL_MAX_COLUMNS_IN_INDEX:
-         case SQL_MAX_COLUMNS_IN_ORDER_BY:
-         case SQL_MAX_COLUMNS_IN_SELECT:
-         case SQL_MAX_COLUMNS_IN_TABLE:
-         case SQL_MAX_CONCURRENT_ACTIVITIES:
-         case SQL_MAX_CURSOR_NAME_LEN:
-         case SQL_MAX_DRIVER_CONNECTIONS:
-         case SQL_MAX_IDENTIFIER_LEN:
-         case SQL_MAX_TABLE_NAME_LEN:
-         case SQL_MAX_TABLES_IN_SELECT:
-         case SQL_MAX_USER_NAME_LEN:
-         case SQL_NON_NULLABLE_COLUMNS:
-         case SQL_NULL_COLLATION:
-         case SQL_ODBC_API_CONFORMANCE:
-         case SQL_ODBC_SAG_CLI_CONFORMANCE:
-         case SQL_ODBC_SQL_CONFORMANCE:
-         case SQL_QUOTED_IDENTIFIER_CASE:
-         case SQL_TXN_CAPABLE:
-
-            retsv = sv_2mortal( newSViv( (I16)( *(SQLSMALLINT*)valuePtr) ) );
-            break;
-
-         /* Create a scalar to hold a 32bit integer */
-         case 2519:                     /* SQL_DATABASE_CODEPAGE:    */
-         case 2520:                     /* SQL_APPLICATION_CODEPAGE: */
-         case 2521:                     /* SQL_CONNECT_CODEPAGE:     */
-         case SQL_ASYNC_MODE:
-         case SQL_BATCH_ROW_COUNT:
-         case SQL_CURSOR_SENSITIVITY:
-         case SQL_DATETIME_LITERALS:
-         case SQL_DDL_INDEX:
-         case SQL_DRIVER_HDBC:
-         case SQL_DRIVER_HDESC:
-         case SQL_DRIVER_HENV:
-         case SQL_DROP_ASSERTION:
-         case SQL_DROP_CHARACTER_SET:
-         case SQL_DROP_COLLATION:
-         case SQL_DROP_DOMAIN:
-         case SQL_DROP_SCHEMA:
-         case SQL_DROP_TABLE:
-         case SQL_DROP_TRANSLATION:
-         case SQL_DROP_VIEW:
-         case SQL_DTC_TRANSITION_COST:
-         case SQL_MAX_ASYNC_CONCURRENT_STATEMENTS:
-         case SQL_MAX_BINARY_LITERAL_LEN:
-         case SQL_MAX_CHAR_LITERAL_LEN:
-         case SQL_MAX_COLUMNS_IN_GROUP_BY:
-         case SQL_MAX_INDEX_SIZE:
-         case SQL_ODBC_INTERFACE_CONFORMANCE:
-         case SQL_PARAM_ARRAY_ROW_COUNTS:
-         case SQL_PARAM_ARRAY_SELECTS:
-         case SQL_SQL_CONFORMANCE:
-
-            retsv = sv_2mortal( newSViv( (I32)( *(SQLINTEGER*)valuePtr) ) );
-            break;
+       /* Create a scalar to hold the string infoTypes */
+       case SQL_DBMS_NAME:
+       case SQL_DBMS_VER:
+       case SQL_CATALOG_NAME:
+       case SQL_CATALOG_NAME_SEPARATOR:
+       case SQL_ACCESSIBLE_PROCEDURES:
+       case SQL_ACCESSIBLE_TABLES:
+       case SQL_CATALOG_TERM:
+       case SQL_COLLATION_SEQ:
+       case SQL_COLUMN_ALIAS:
+       case SQL_DATA_SOURCE_NAME:
+       case SQL_DATA_SOURCE_READ_ONLY:
+       case SQL_DATABASE_NAME:
+       case SQL_DESCRIBE_PARAMETER:
+       case SQL_DRIVER_NAME:
+       case SQL_DRIVER_ODBC_VER:
+       case SQL_DRIVER_VER:
+       case SQL_EXPRESSIONS_IN_ORDERBY:
+       case SQL_IDENTIFIER_QUOTE_CHAR:
+       case SQL_INTEGRITY:
+       case SQL_KEYWORDS:
+       case SQL_LIKE_ESCAPE_CLAUSE:
+       case SQL_MULT_RESULT_SETS:
+       case SQL_MULTIPLE_ACTIVE_TXN:
+       case SQL_NEED_LONG_DATA_LEN:
+       case SQL_ODBC_VER:
+       case SQL_ORDER_BY_COLUMNS_IN_SELECT:
+       case SQL_OUTER_JOINS:
+       case SQL_PROCEDURE_TERM:
+       case SQL_PROCEDURES:
+       case SQL_ROW_UPDATES:
+       case SQL_SCHEMA_TERM:
+       case SQL_SEARCH_PATTERN_ESCAPE:
+       case SQL_SERVER_NAME:
+       case SQL_SPECIAL_CHARACTERS:
+       case SQL_TABLE_TERM:
+       case SQL_USER_NAME:
+       case SQL_XOPEN_CLI_YEAR:
 
 
-         /* Createa scalar to hold the 32-bit mask */
-         /* not supported */
-         case SQL_AGGREGATE_FUNCTIONS:
-         case SQL_ALTER_DOMAIN:
-         case SQL_ALTER_TABLE:
-         case SQL_BATCH_SUPPORT:
-         case SQL_BOOKMARK_PERSISTENCE:
-         case SQL_CATALOG_USAGE:
-         case SQL_CONVERT_BIGINT:
-         case SQL_CONVERT_BINARY:
-         case SQL_CONVERT_BIT:
-         case SQL_CONVERT_CHAR:
-         case SQL_CONVERT_DATE:
-         case SQL_CONVERT_DECIMAL:
-         case SQL_CONVERT_DOUBLE:
-         case SQL_CONVERT_FLOAT:
-         case SQL_CONVERT_INTEGER:
-         case SQL_CONVERT_INTERVAL_YEAR_MONTH:
-         case SQL_CONVERT_INTERVAL_DAY_TIME:
-         case SQL_CONVERT_LONGVARBINARY:
-         case SQL_CONVERT_LONGVARCHAR:
-         case SQL_CONVERT_NUMERIC:
-         case SQL_CONVERT_REAL:
-         case SQL_CONVERT_SMALLINT:
-         case SQL_CONVERT_TIME:
-         case SQL_CONVERT_TIMESTAMP:
-         case SQL_CONVERT_TINYINT:
-         case SQL_CONVERT_VARBINARY:
-         case SQL_CONVERT_VARCHAR:
-         case SQL_CONVERT_WCHAR:
-         case SQL_CONVERT_WLONGVARCHAR:
-         case SQL_CONVERT_WVARCHAR:
-         case SQL_CONVERT_FUNCTIONS:
-         case SQL_CREATE_ASSERTION:
-         case SQL_CREATE_CHARACTER_SET:
-         case SQL_CREATE_COLLATION:
-         case SQL_CREATE_DOMAIN:
-         case SQL_CREATE_SCHEMA:
-         case SQL_CREATE_TABLE:
-         case SQL_CREATE_TRANSLATION:
-         case SQL_CREATE_VIEW:
-         case SQL_DEFAULT_TXN_ISOLATION:
-         /*case SQL_DYNAMIC_CURSOR_ATTRIBUTES:*/
-         case SQL_DYNAMIC_CURSOR_ATTRIBUTES2:
-         case SQL_FETCH_DIRECTION:
-         case SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1:
-         case SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2:
-         case SQL_GETDATA_EXTENSIONS:
-         case SQL_INDEX_KEYWORDS:
-         case SQL_INFO_SCHEMA_VIEWS:
-         case SQL_INSERT_STATEMENT:
-         case SQL_KEYSET_CURSOR_ATTRIBUTES1:
-         case SQL_KEYSET_CURSOR_ATTRIBUTES2:
-         case SQL_LOCK_TYPES:
-         case SQL_NUMERIC_FUNCTIONS:
-         case SQL_OJ_CAPABILITIES:
-         case SQL_POS_OPERATIONS:
-         case SQL_POSITIONED_STATEMENTS:
-         case SQL_SCHEMA_USAGE:
-         case SQL_SCROLL_CONCURRENCY:
-         case SQL_SCROLL_OPTIONS:
-         case SQL_SQL92_DATETIME_FUNCTIONS:
-         case SQL_SQL92_FOREIGN_KEY_DELETE_RULE:
-         case SQL_SQL92_FOREIGN_KEY_UPDATE_RULE:
-         case SQL_SQL92_GRANT:
-         case SQL_SQL92_NUMERIC_VALUE_FUNCTIONS:
-         case SQL_SQL92_PREDICATES:
-         case SQL_SQL92_RELATIONAL_JOIN_OPERATORS:
-         case SQL_SQL92_REVOKE:
-         case SQL_SQL92_ROW_VALUE_CONSTRUCTOR:
-         case SQL_SQL92_STRING_FUNCTIONS:
-         case SQL_SQL92_VALUE_EXPRESSIONS:
-         case SQL_STANDARD_CLI_CONFORMANCE:
-         case SQL_STATIC_CURSOR_ATTRIBUTES1:
-         case SQL_STATIC_CURSOR_ATTRIBUTES2:
-         case SQL_STATIC_SENSITIVITY:
-         case SQL_STRING_FUNCTIONS:
-         case SQL_SUBQUERIES:
-         case SQL_SYSTEM_FUNCTIONS:
-         case SQL_TIMEDATE_ADD_INTERVALS:
-         case SQL_TIMEDATE_DIFF_INTERVALS:
-         case SQL_TIMEDATE_FUNCTIONS:
-         case SQL_TXN_ISOLATION_OPTION:
-         case SQL_UNION:
+	     ret = SQLGetInfo( imp_dbh->hdbc,
+                           infoType,
+                           valuePtr,
+                           bufferLength,
+                           &stringLength );
 
-            /* how do return a bitmask? */
-            /* retsv = sv_2mortal( newSVpv( (char*) valuePtr, (int) 32 ) ); */
-            /* retsv = sv_2mortal( newSVrv( (SV*) valuePtr, NULL ) ); */
-            break;
+         if( ret == SQL_SUCCESS_WITH_INFO &&  bufferLength < (stringLength + 1) )
+	     {
+	       if( DBIS->debug >= 2)
+	       {
+		     PerlIO_printf( DBILOGFP,
+		       "GetInfo(%d) local buffer isn't big enough. stringlenght=%d\n",
+		       infoType, stringLength );
+	       }
+
+	       /* Local buffer isn't big enough, dynamically allocate new one */
+	       bufferLength = stringLength + 1;
+	       Newc( 1, valuePtr, bufferLength, char, SQLPOINTER );
+	       Zero( valuePtr, bufferLength, char );
+
+	       ret = SQLGetInfo( imp_dbh->hdbc,
+                             infoType,
+                             valuePtr,
+                             bufferLength,
+                             &stringLength );
+	     }
+
+	     ret = check_error( dbh, ret, "Error calling SQLGetInfo" );
+
+         if( ret == SQL_SUCCESS){
+           retsv = sv_2mortal( newSVpvn( (char*) valuePtr, (int)stringLength ) );
+           break;
+         }
+
+       /* Create a scalar to hold the 16-bit integer */
+       case SQL_CATALOG_LOCATION:
+       case SQL_CONCAT_NULL_BEHAVIOR:
+       case SQL_CORRELATION_NAME:
+       case SQL_CURSOR_COMMIT_BEHAVIOR:
+       case SQL_CURSOR_ROLLBACK_BEHAVIOR:
+       case SQL_FILE_USAGE:
+       case SQL_GROUP_BY:
+       case SQL_IDENTIFIER_CASE:
+       case SQL_MAX_CATALOG_NAME_LEN:
+       case SQL_MAX_COLUMN_NAME_LEN:
+       case SQL_MAX_COLUMNS_IN_INDEX:
+       case SQL_MAX_COLUMNS_IN_ORDER_BY:
+       case SQL_MAX_COLUMNS_IN_SELECT:
+       case SQL_MAX_COLUMNS_IN_TABLE:
+       case SQL_MAX_CONCURRENT_ACTIVITIES:
+       case SQL_MAX_CURSOR_NAME_LEN:
+       case SQL_MAX_DRIVER_CONNECTIONS:
+       case SQL_MAX_IDENTIFIER_LEN:
+       case SQL_MAX_TABLE_NAME_LEN:
+       case SQL_MAX_TABLES_IN_SELECT:
+       case SQL_MAX_USER_NAME_LEN:
+       case SQL_NON_NULLABLE_COLUMNS:
+       case SQL_NULL_COLLATION:
+       case SQL_ODBC_API_CONFORMANCE:
+       case SQL_ODBC_SAG_CLI_CONFORMANCE:
+       case SQL_ODBC_SQL_CONFORMANCE:
+       case SQL_QUOTED_IDENTIFIER_CASE:
+       case SQL_TXN_CAPABLE:
+
+	     ret = SQLGetInfo( imp_dbh->hdbc,
+                           infoType,
+                           valuePtr,
+                           bufferLength,
+                           &stringLength );
+
+         if( ret == SQL_SUCCESS_WITH_INFO &&  bufferLength < (stringLength + 1) )
+	     {
+	       if( DBIS->debug >= 2)
+	       {
+		     PerlIO_printf( DBILOGFP,
+		       "GetInfo(%d) local buffer isn't big enough. stringlenght=%d\n",
+		       infoType, stringLength );
+	       }
+
+	       /* Local buffer isn't big enough, dynamically allocate new one */
+	       bufferLength = stringLength + 1;
+	       Newc( 1, valuePtr, bufferLength, char, SQLPOINTER );
+	       Zero( valuePtr, bufferLength, char );
+
+	       ret = SQLGetInfo( imp_dbh->hdbc,
+                             infoType,
+                             valuePtr,
+                             bufferLength,
+                             &stringLength );
+	     }
+
+	     ret = check_error( dbh, ret, "Error calling SQLGetInfo" );
+
+         if( ret == SQL_SUCCESS){
+           retsv = sv_2mortal( newSViv( (I16)( *(SQLSMALLINT*)valuePtr) ) );  
+           break;
+         }
+
+       /* Create a scalar to hold a 32bit integer */
+       case 2519:                     /* SQL_DATABASE_CODEPAGE:    */
+       case 2520:                     /* SQL_APPLICATION_CODEPAGE: */
+       case 2521:                     /* SQL_CONNECT_CODEPAGE:     */
+       case SQL_ASYNC_MODE:
+       case SQL_BATCH_ROW_COUNT:
+       case SQL_CURSOR_SENSITIVITY:
+       case SQL_DATETIME_LITERALS:
+       case SQL_DDL_INDEX:
+       case SQL_DRIVER_HDBC:
+       case SQL_DRIVER_HDESC:
+       case SQL_DRIVER_HENV:
+       case SQL_DROP_ASSERTION:
+       case SQL_DROP_CHARACTER_SET:
+       case SQL_DROP_COLLATION:
+       case SQL_DROP_DOMAIN:
+       case SQL_DROP_SCHEMA:
+       case SQL_DROP_TABLE:
+       case SQL_DROP_TRANSLATION:
+       case SQL_DROP_VIEW:
+       case SQL_DTC_TRANSITION_COST:
+       case SQL_MAX_ASYNC_CONCURRENT_STATEMENTS:
+       case SQL_MAX_BINARY_LITERAL_LEN:
+       case SQL_MAX_CHAR_LITERAL_LEN:
+       case SQL_MAX_COLUMNS_IN_GROUP_BY:
+       case SQL_MAX_INDEX_SIZE:
+       case SQL_ODBC_INTERFACE_CONFORMANCE:
+       case SQL_PARAM_ARRAY_ROW_COUNTS:
+       case SQL_PARAM_ARRAY_SELECTS:
+       case SQL_SQL_CONFORMANCE:
 
 
-         default:
-            break;
+	     ret = SQLGetInfo( imp_dbh->hdbc,
+                           infoType,
+                           valuePtr,
+                           bufferLength,
+                           &stringLength );
+
+         if( ret == SQL_SUCCESS_WITH_INFO &&  bufferLength < (stringLength + 1) )
+	     {
+	       if( DBIS->debug >= 2)
+	       {
+		     PerlIO_printf( DBILOGFP,
+		       "GetInfo(%d) local buffer isn't big enough. stringlenght=%d\n",
+		       infoType, stringLength );
+	       }
+
+	       /* Local buffer isn't big enough, dynamically allocate new one */
+	       bufferLength = stringLength + 1;
+	       Newc( 1, valuePtr, bufferLength, char, SQLPOINTER );
+	       Zero( valuePtr, bufferLength, char );
+
+	       ret = SQLGetInfo( imp_dbh->hdbc,
+                             infoType,
+                             valuePtr,
+                             bufferLength,
+                             &stringLength );
+	     }
+
+	     ret = check_error( dbh, ret, "Error calling SQLGetInfo" );
+
+         if( ret == SQL_SUCCESS){
+           retsv = sv_2mortal( newSViv( (I32)( *(SQLINTEGER*)valuePtr) ) );
+           break;
+         }
+
+       /* Createa scalar to hold the 32-bit mask */
+       /* not supported */
+       case SQL_AGGREGATE_FUNCTIONS:
+       case SQL_ALTER_DOMAIN:
+       case SQL_ALTER_TABLE:
+       case SQL_BATCH_SUPPORT:
+       case SQL_BOOKMARK_PERSISTENCE:
+       case SQL_CATALOG_USAGE:
+       case SQL_CONVERT_BIGINT:
+       case SQL_CONVERT_BINARY:
+       case SQL_CONVERT_BIT:
+       case SQL_CONVERT_CHAR:
+       case SQL_CONVERT_DATE:
+       case SQL_CONVERT_DECIMAL:
+       case SQL_CONVERT_DOUBLE:
+       case SQL_CONVERT_FLOAT:
+       case SQL_CONVERT_INTEGER:
+       case SQL_CONVERT_INTERVAL_YEAR_MONTH:
+       case SQL_CONVERT_INTERVAL_DAY_TIME:
+       case SQL_CONVERT_LONGVARBINARY:
+       case SQL_CONVERT_LONGVARCHAR:
+       case SQL_CONVERT_NUMERIC:
+       case SQL_CONVERT_REAL:
+       case SQL_CONVERT_SMALLINT:
+       case SQL_CONVERT_TIME:
+       case SQL_CONVERT_TIMESTAMP:
+       case SQL_CONVERT_TINYINT:
+       case SQL_CONVERT_VARBINARY:
+       case SQL_CONVERT_VARCHAR:
+       case SQL_CONVERT_WCHAR:
+       case SQL_CONVERT_WLONGVARCHAR:
+       case SQL_CONVERT_WVARCHAR:
+       case SQL_CONVERT_FUNCTIONS:
+       case SQL_CREATE_ASSERTION:
+       case SQL_CREATE_CHARACTER_SET:
+       case SQL_CREATE_COLLATION:
+       case SQL_CREATE_DOMAIN:
+       case SQL_CREATE_SCHEMA:
+       case SQL_CREATE_TABLE:
+       case SQL_CREATE_TRANSLATION:
+       case SQL_CREATE_VIEW:
+       case SQL_DEFAULT_TXN_ISOLATION:
+       /*case SQL_DYNAMIC_CURSOR_ATTRIBUTES:*/
+       case SQL_DYNAMIC_CURSOR_ATTRIBUTES2:
+       case SQL_FETCH_DIRECTION:
+       case SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1:
+       case SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2:
+       case SQL_GETDATA_EXTENSIONS:
+       case SQL_INDEX_KEYWORDS:
+       case SQL_INFO_SCHEMA_VIEWS:
+       case SQL_INSERT_STATEMENT:
+       case SQL_KEYSET_CURSOR_ATTRIBUTES1:
+       case SQL_KEYSET_CURSOR_ATTRIBUTES2:
+       case SQL_LOCK_TYPES:
+       case SQL_NUMERIC_FUNCTIONS:
+       case SQL_OJ_CAPABILITIES:
+       case SQL_POS_OPERATIONS:
+       case SQL_POSITIONED_STATEMENTS:
+       case SQL_SCHEMA_USAGE:
+       case SQL_SCROLL_CONCURRENCY:
+       case SQL_SCROLL_OPTIONS:
+       case SQL_SQL92_DATETIME_FUNCTIONS:
+       case SQL_SQL92_FOREIGN_KEY_DELETE_RULE:
+       case SQL_SQL92_FOREIGN_KEY_UPDATE_RULE:
+       case SQL_SQL92_GRANT:
+       case SQL_SQL92_NUMERIC_VALUE_FUNCTIONS:
+       case SQL_SQL92_PREDICATES:
+       case SQL_SQL92_RELATIONAL_JOIN_OPERATORS:
+       case SQL_SQL92_REVOKE:
+       case SQL_SQL92_ROW_VALUE_CONSTRUCTOR:
+       case SQL_SQL92_STRING_FUNCTIONS:
+       case SQL_SQL92_VALUE_EXPRESSIONS:
+       case SQL_STANDARD_CLI_CONFORMANCE:
+       case SQL_STATIC_CURSOR_ATTRIBUTES1:
+       case SQL_STATIC_CURSOR_ATTRIBUTES2:
+       case SQL_STATIC_SENSITIVITY:
+       case SQL_STRING_FUNCTIONS:
+       case SQL_SUBQUERIES:
+       case SQL_SYSTEM_FUNCTIONS:
+       case SQL_TIMEDATE_ADD_INTERVALS:
+       case SQL_TIMEDATE_DIFF_INTERVALS:
+       case SQL_TIMEDATE_FUNCTIONS:
+       case SQL_TXN_ISOLATION_OPTION:
+       case SQL_UNION:
+
+         /* how do return a bitmask? */
+         /* retsv = sv_2mortal( newSVpv( (char*) valuePtr, (int) 32 ) ); */
+         /* retsv = sv_2mortal( newSVrv( (SV*) valuePtr, NULL ) ); */
+
+         break;
+
+
+       default:
+         break;
       }
-   }
 
    /* Free dynamically allocated buffer */
    if( valuePtr != (SQLPOINTER) buffer )
@@ -2744,6 +2849,20 @@ int dbd_st_rows( SV *sth,
     return imp_sth->RowCount;
 }
 
+int dbd_st_cancel( SV *sth,
+                   imp_sth_t *imp_sth )
+{
+    D_imp_dbh_from_sth;
+    SQLRETURN ret;
+
+    if (DBIc_ACTIVE(imp_sth) ) {
+        ret = SQLCancel(imp_sth->phstmt);
+        ret = check_error( sth, ret, "SQLCancel failed" );
+        EOI(ret);
+    }
+    DBIc_ACTIVE_off(imp_sth);
+    return 1;
+}
 
 int dbd_st_finish( SV *sth,
                    imp_sth_t *imp_sth )
